'''

'''

class BioSequence(object):
    '''
    BioSequence class is the base class for biological
    sequence representation.
    
    It provides storage of the sequence itself, of an identifier,
    a defintion an manage a set of complementary information on
    a key / value principle.
    
    BioSequence is an abstract class and must be instanciate 
    from its subclasses
    '''
    
    def __init__(self,id,seq,definition=None,**info):
        '''
        BioSequence constructor.
        
        @param id: sequence identifier
        @type id:  str
        @param seq: the sequence
        @type seq:  str
        @param definition: sequence defintion (optional)
        @type definition: str
        
        extra named parametters can be add to associtiate complementary
        data to the sequence
        '''
        
        self.seq = seq
        self.definition = definition
        self.id = id
        self._info = dict(info)
        
    def __str__(self):
        return self.seq
    
    def __getitem__(self,key):
        if isinstance(key, str):
            return self._info[key]
        elif isinstance(key, int):
            return self.seq[key]
        elif isinstance(key, slice):
            subseq=self.seq[key]
            info = dict(self._info)
            if key.start is not None:
                start = key.start +1
            else:
                start = 1
            if key.stop is not None:
                stop = key.stop+1
            else:
                stop = len(self.seq)
            if key.step is not None:
                step = key.step
            else:
                step = 1
            
            info['cut']='[%d,%d,%s]' % (start,stop,step)
            return bioSeqGenerator(self.id, subseq, self.definition,**info)
        raise TypeError,'key must be an integer, a str or a slice'  
        
    def __setitem__(self,key,value):
        self._info[key]=value
        
    def __iter__(self):
        return iter(self.seq)
    
    def __len__(self):
        return len(self.seq)
    
    def isNucleotide(self):
        raise NotImplementedError
        
class NucSequence(BioSequence):
    _comp={'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A',
           'R': 'Y', 'Y': 'R', 'K': 'M', 'M': 'K', 
           'S': 'S', 'W': 'W', 'B': 'V', 'D': 'H', 
           'H': 'D', 'V': 'B', 'N': 'N', 'U': 'A',
           '-': '-'}
 
    def complement(self):
        cseq = [NucSequence._comp.get(x,'N') for x in self.seq]
        cseq.reverse()
        rep = NucSequence(self.id,''.join(cseq),self.definition,**self._info)
        rep._info['complemented']=not rep._info.get('complemented',False)
        return rep
    
    def isNucleotide(self):
        return True
    
class AASequence(BioSequence):

    def isNucleotide(self):
        return False
    
    
    
def _isNucSeq(text):
    acgt   = 0
    notnuc = 0
    ltot   = len(text)
    for c in text.upper():
        if c in 'ACGT-':
            acgt+=1
        if c not in NucSequence._comp:
            notnuc+=1
    return notnuc==0 and float(acgt)/ltot > 0.8


def bioSeqGenerator(id,seq,definition=None,**info):
    if _isNucSeq(seq):
        return NucSequence(id,seq,definition,**info)
    else:
        return AASequence(id,seq,definition,**info)